// Resource history and resource pipeline.

"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _createClass = require("babel-runtime/helpers/create-class")["default"];

var _inherits = require("babel-runtime/helpers/inherits")["default"];

var _slicedToArray = require("babel-runtime/helpers/sliced-to-array")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _regeneratorRuntime = require("babel-runtime/regenerator")["default"];

var _ = require("lodash");
var assert = require("assert");
var DOM = require("./dom");
var File = require("fs");
var Fetch = require("./fetch");

var _require = require("./fetch");

var Headers = _require.Headers;

var _require2 = require("util");

var isArray = _require2.isArray;

var Path = require("path");
var Request = require("request");
var URL = require("url");
var Utils = require("jsdom/lib/jsdom/utils");

var Resource = (function () {
  function Resource(_ref) {
    var request = _ref.request;

    _classCallCheck(this, Resource);

    this.request = request;
    this.error = null;
    this.response = null;
  }

  Resource.prototype.dump = function dump(output) {
    var _ref2 = this;

    var request = _ref2.request;
    var response = _ref2.response;
    var error = _ref2.error;

    // Write summary request/response header
    if (response) {
      var elapsed = response.time - request.time;
      output.write("" + request.method + " " + this.url + " - " + response.status + " " + response.statusText + " - " + elapsed + "ms\n");
    } else output.write("" + request.method + " " + this.url + "\n");

    // If response, write out response headers and sample of document entity
    // If error, write out the error message
    // Otherwise, indicate this is a pending request
    if (response) {
      if (response._redirectCount) output.write("  Followed " + response._redirectCount + " redirects\n");
      for (var _iterator = response.headers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var _ref3 = _slicedToArray(_ref, 2);

        var _name = _ref3[0];
        var value = _ref3[1];

        output.write("  " + _name + ": " + value + "\n");
      }output.write("\n");
      var sample = response.body.slice(0, 250).toString("utf8").split("\n").map(function (line) {
        return "  " + line;
      }).join("\n");
      output.write(sample);
    } else if (error) output.write("  Error: " + error.message + "\n");else output.write("  Pending since " + new Date(request.time) + "\n");
    // Keep them separated
    output.write("\n\n");
  };

  _createClass(Resource, {
    url: {
      get: function () {
        return this.response && this.response.url || this.request.url;
      }
    }
  });

  return Resource;
})();

// Each browser has a resources object that provides the means for retrieving
// resources and a list of all retrieved resources.
//
// The object is an array, and its elements are the resources.

var Resources = (function (_Array) {
  function Resources(browser) {
    _classCallCheck(this, Resources);

    this.browser = browser;
    this.pipeline = Resources.pipeline.slice();
  }

  _inherits(Resources, _Array);

  Resources.prototype.fetch = function fetch(input, init) {
    var _this = this;

    var request, resource, response;
    return _regeneratorRuntime.async(function fetch$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          request = new Fetch.Request(input, init);
          resource = new Resource({ request: request });

          _this.push(resource);
          _this.browser.emit("request", request);

          context$2$0.prev = 4;
          context$2$0.next = 7;
          return _this._runPipeline(request);

        case 7:
          response = context$2$0.sent;

          if (!response) {
            context$2$0.next = 16;
            break;
          }

          response.time = Date.now();
          response.request = request;
          resource.response = response;
          _this.browser.emit("response", request, response);
          return context$2$0.abrupt("return", response);

        case 16:
          resource.response = Fetch.Resource.error();
          throw new TypeError("No response");

        case 18:
          context$2$0.next = 26;
          break;

        case 20:
          context$2$0.prev = 20;
          context$2$0.t2 = context$2$0["catch"](4);

          _this.browser._debug("Resource error", context$2$0.t2.stack);
          resource.error = context$2$0.t2;
          resource.response = Fetch.Response.error();
          throw new TypeError(context$2$0.t2.message);

        case 26:
        case "end":
          return context$2$0.stop();
      }
    }, null, this, [[4, 20]]);
  };

  // Human readable resource listing.
  //
  // output - Write to this stream (optional)

  Resources.prototype.dump = function dump() {
    var output = arguments[0] === undefined ? process.stdout : arguments[0];

    if (this.length === 0) output.write("No resources\n");else this.forEach(function (resource) {
      return resource.dump(output);
    });
  };

  // Add a request/response handler.  This handler will only be used by this
  // browser.

  Resources.prototype.addHandler = function addHandler(handler) {
    assert(handler.call, "Handler must be a function");
    assert(handler.length === 2 || handler.length === 3, "Handler function takes 2 (request handler) or 3 (reponse handler) arguments");
    this.pipeline.push(handler);
  };

  // Processes the request using the pipeline.

  Resources.prototype._runPipeline = function _runPipeline(request) {
    var _this = this;

    var _ref3, browser, requestHandlers, responseHandlers, response, _iterator, _isArray, _i, _ref, requestHandler, _iterator2, _isArray2, _i2, _ref2, responseHandler;

    return _regeneratorRuntime.async(function _runPipeline$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          _ref3 = _this;
          browser = _ref3.browser;
          requestHandlers = _this.pipeline.filter(function (fn) {
            return fn.length === 2;
          }).concat(Resources.makeHTTPRequest);
          responseHandlers = _this.pipeline.filter(function (fn) {
            return fn.length === 3;
          });
          response = undefined;
          _iterator = requestHandlers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);

        case 6:
          if (!_isArray) {
            context$2$0.next = 12;
            break;
          }

          if (!(_i >= _iterator.length)) {
            context$2$0.next = 9;
            break;
          }

          return context$2$0.abrupt("break", 24);

        case 9:
          _ref = _iterator[_i++];
          context$2$0.next = 16;
          break;

        case 12:
          _i = _iterator.next();

          if (!_i.done) {
            context$2$0.next = 15;
            break;
          }

          return context$2$0.abrupt("break", 24);

        case 15:
          _ref = _i.value;

        case 16:
          requestHandler = _ref;
          context$2$0.next = 19;
          return requestHandler(browser, request);

        case 19:
          response = context$2$0.sent;

          if (!response) {
            context$2$0.next = 22;
            break;
          }

          return context$2$0.abrupt("break", 24);

        case 22:
          context$2$0.next = 6;
          break;

        case 24:
          _iterator2 = responseHandlers, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _core.$for.getIterator(_iterator2);

        case 25:
          if (!_isArray2) {
            context$2$0.next = 31;
            break;
          }

          if (!(_i2 >= _iterator2.length)) {
            context$2$0.next = 28;
            break;
          }

          return context$2$0.abrupt("break", 41);

        case 28:
          _ref2 = _iterator2[_i2++];
          context$2$0.next = 35;
          break;

        case 31:
          _i2 = _iterator2.next();

          if (!_i2.done) {
            context$2$0.next = 34;
            break;
          }

          return context$2$0.abrupt("break", 41);

        case 34:
          _ref2 = _i2.value;

        case 35:
          responseHandler = _ref2;
          context$2$0.next = 38;
          return responseHandler(browser, request, response);

        case 38:
          response = context$2$0.sent;

        case 39:
          context$2$0.next = 25;
          break;

        case 41:
          return context$2$0.abrupt("return", response);

        case 42:
        case "end":
          return context$2$0.stop();
      }
    }, null, this);
  };

  // -- Handlers --

  Resources.addHandler = function addHandler(handler) {
    assert(handler.call, "Handler must be a function");
    assert(handler.length === 2 || handler.length === 3, "Handler function takes 2 (request handler) or 3 (response handler) arguments");
    this.pipeline.push(handler);
  };

  // This handler normalizes the request URL.
  //
  // It turns relative URLs into absolute URLs based on the current document URL
  // or base element, or if no document open, based on browser.site property.

  Resources.normalizeURL = function normalizeURL(browser, request) {
    if (browser.document)
      // Resolve URL relative to document URL/base, or for new browser, using
      // Browser.site
      request.url = DOM.resourceLoader.resolve(browser.document, request.url);else request.url = Utils.resolveHref(browser.site || "http://localhost", request.url);
  };

  // This handler mergers request headers.
  //
  // It combines headers provided in the request with custom headers defined by
  // the browser (user agent, authentication, etc).
  //
  // It also normalizes all headers by down-casing the header names.

  Resources.mergeHeaders = function mergeHeaders(browser, request) {
    if (browser.headers) _.each(browser.headers, function (value, name) {
      request.headers.append(name, browser.headers[name]);
    });
    if (!request.headers.has("User-Agent")) request.headers.set("User-Agent", browser.userAgent);

    // Always pass Host: from request URL

    var _URL$parse = URL.parse(request.url);

    var host = _URL$parse.host;

    request.headers.set("Host", host);

    // HTTP Basic authentication
    var authenticate = { host: host, username: null, password: null };
    browser.emit("authenticate", authenticate);
    var username = authenticate.username;
    var password = authenticate.password;

    if (username && password) {
      browser.log("Authenticating as " + username + ":" + password);
      var base64 = new Buffer("" + username + ":" + password).toString("base64");
      request.headers.set("authorization", "Basic " + base64);
    }
  };

  // Used to perform HTTP request (also supports file: resources).  This is always
  // the last request handler.

  Resources.makeHTTPRequest = function makeHTTPRequest(browser, request) {
    var _this = this;

    var url, _URL$parse, protocol, hostname, pathname, filename, exists, stream, _ret;

    return _regeneratorRuntime.async(function makeHTTPRequest$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          url = request.url;
          _URL$parse = URL.parse(url);
          protocol = _URL$parse.protocol;
          hostname = _URL$parse.hostname;
          pathname = _URL$parse.pathname;

          if (!(protocol === "file:")) {
            context$2$0.next = 18;
            break;
          }

          if (!(request.method !== "GET")) {
            context$2$0.next = 8;
            break;
          }

          return context$2$0.abrupt("return", new Fetch.Response("", { url: url, status: 405 }));

        case 8:
          filename = Path.normalize(decodeURI(pathname));
          exists = File.existsSync(filename);

          if (!exists) {
            context$2$0.next = 15;
            break;
          }

          stream = File.createReadStream(filename);
          return context$2$0.abrupt("return", new Fetch.Response(stream, { url: url, status: 200 }));

        case 15:
          return context$2$0.abrupt("return", new Fetch.Response("", { url: url, status: 404 }));

        case 16:
          context$2$0.next = 23;
          break;

        case 18:
          context$2$0.next = 20;
          return (function callee$2$0() {
            var cookies, cookieHeader, buffer, httpRequest;
            return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
              while (1) switch (context$3$0.prev = context$3$0.next) {
                case 0:
                  cookies = browser.cookies;
                  cookieHeader = cookies.serialize(hostname, pathname);

                  if (cookieHeader) request.headers.append("Cookie", cookieHeader);

                  if (!/^GET|HEAD$/.test(request.method)) {
                    context$3$0.next = 7;
                    break;
                  }

                  context$3$0.t3 = null;
                  context$3$0.next = 10;
                  break;

                case 7:
                  context$3$0.next = 9;
                  return request._consume();

                case 9:
                  context$3$0.t3 = context$3$0.sent;

                case 10:
                  buffer = context$3$0.t3;
                  httpRequest = new Request({
                    method: request.method,
                    uri: request.url,
                    headers: request.headers.toObject(),
                    proxy: browser.proxy,
                    body: buffer,
                    jar: false,
                    followRedirect: false,
                    strictSSL: browser.strictSSL,
                    localAddress: browser.localAddress || 0
                  });
                  context$3$0.next = 14;
                  return new _core.Promise(function (resolve, reject) {
                    httpRequest.on("response", function (response) {
                      // Request returns an object where property name is header name,
                      // property value is either header value, or an array if header sent
                      // multiple times (e.g. `Set-Cookie`).
                      var arrayOfHeaders = _.reduce(response.headers, function (headers, value, name) {
                        if (isArray(value)) for (var _iterator = value, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
                            var _ref;

                            if (_isArray) {
                              if (_i >= _iterator.length) break;
                              _ref = _iterator[_i++];
                            } else {
                              _i = _iterator.next();
                              if (_i.done) break;
                              _ref = _i.value;
                            }

                            var item = _ref;

                            headers.push([name, item]);
                          } else headers.push([name, value]);
                        return headers;
                      }, []);

                      resolve(new Fetch.Response(response, {
                        url: request.url,
                        status: response.statusCode,
                        headers: new Headers(arrayOfHeaders)
                      }));
                    }).on("error", reject);
                  });

                case 14:
                  context$3$0.t4 = context$3$0.sent;
                  return context$3$0.abrupt("return", {
                    v: context$3$0.t4
                  });

                case 16:
                case "end":
                  return context$3$0.stop();
              }
            }, null, _this);
          })();

        case 20:
          _ret = context$2$0.sent;

          if (!(typeof _ret === "object")) {
            context$2$0.next = 23;
            break;
          }

          return context$2$0.abrupt("return", _ret.v);

        case 23:
        case "end":
          return context$2$0.stop();
      }
    }, null, this);
  };

  Resources.handleHeaders = function handleHeaders(browser, request, response) {
    response.headers = new Headers(response.headers);
    return response;
  };

  Resources.handleCookies = function handleCookies(browser, request, response) {
    // Set cookies from response: call update() with array of headers

    var _URL$parse = URL.parse(request.url);

    var hostname = _URL$parse.hostname;
    var pathname = _URL$parse.pathname;

    var newCookies = response.headers.getAll("Set-Cookie");
    browser.cookies.update(newCookies, hostname, pathname);
    return response;
  };

  Resources.handleRedirect = function handleRedirect(browser, request, response) {
    var status, _location;

    return _regeneratorRuntime.async(function handleRedirect$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          status = response.status;

          if (!(status === 301 || status === 302 || status === 303 || status === 307 || status === 308)) {
            context$2$0.next = 19;
            break;
          }

          if (!(request.redirect === "error")) {
            context$2$0.next = 4;
            break;
          }

          return context$2$0.abrupt("return", Fetch.Response.error());

        case 4:
          _location = response.headers.get("Location");

          if (!(_location === null)) {
            context$2$0.next = 7;
            break;
          }

          return context$2$0.abrupt("return", response);

        case 7:
          if (!(request._redirectCount >= 20)) {
            context$2$0.next = 9;
            break;
          }

          return context$2$0.abrupt("return", Fetch.Response.error());

        case 9:

          browser.emit("redirect", request, response, _location);
          ++request._redirectCount;
          if (status !== 307) {
            request.method = "GET";
            request.headers["delete"]("Content-Type");
            request.headers["delete"]("Content-Length");
            request.headers["delete"]("Content-Transfer-Encoding");
          }

          // This request is referer for next
          request.headers.set("Referer", request.url);
          request.url = Utils.resolveHref(request.url, _location);
          context$2$0.next = 16;
          return browser.resources._runPipeline(request);

        case 16:
          return context$2$0.abrupt("return", context$2$0.sent);

        case 19:
          return context$2$0.abrupt("return", response);

        case 20:
        case "end":
          return context$2$0.stop();
      }
    }, null, this);
  };

  return Resources;
})(Array);

// All browsers start out with this list of handler.
Resources.pipeline = [Resources.normalizeURL, Resources.mergeHeaders, Resources.handleHeaders, Resources.handleCookies, Resources.handleRedirect];

module.exports = Resources;

// If the request is for a file:// descriptor, just open directly from the
// file system rather than getting node's http (which handles file://
// poorly) involved.

// We're going to use cookies later when recieving response.
//# sourceMappingURL=resources.js.map